import {Component, EventEmitter, Output} from '@angular/core';
import {SelectionModel} from "@angular/cdk/collections";
import {HttpClient} from "@angular/common/http";
import {ToastrService} from "ngx-toastr";
import {ConfigService} from "@services/config.service";
import {ActivatedRoute, Router} from "@angular/router";
import {SaveScriptComponent} from "./save-script/save-script.component";
import {MatDialog} from "@angular/material/dialog";

@Component({
  selector: 'app-run-script-assistant',
  templateUrl: './run-script-assistant.component.html',
  styleUrl: './run-script-assistant.component.css'
})
export class RunScriptAssistantComponent {
  baseUrl: string;
  @Output() dataChange = new EventEmitter<any>();

  errorMessage = '';
  clientId: string | null = null;
  name: string = '';
  client: any = null;
  clientData: any = [];
  loading: boolean = false;
  scripts: any[] = [];
  scriptContent: string = "";
  parameters: any = {};
  selectedScript: any = null;
  selectedClients: any[] = [];
  allSelected: boolean = true;
  commandType: string = 'existing';
  newScript: string = '';
  selection = new SelectionModel(true, []);
  parameterNames: string[] = Object.keys(this.parameters);

  constructor(
    private http: HttpClient,
    private toastService: ToastrService,
    private configService: ConfigService,
    private router: Router,
    private dialog: MatDialog,
    private route: ActivatedRoute
  ) {
    this.baseUrl = this.configService.apiUrl;
    this.route.queryParams.subscribe(params => {
      if (params['clientData']) {
        this.clientData = JSON.parse(params['clientData']);
      }
    });
    this.clientId = this.clientData?.length ? this.clientData[0]['@id'] : null;
    this.clientData.forEach((client: { selected: boolean; status: string}) => {
      if (client.status === 'og-live') {
        client.selected = true;
      }
    });
    this.selectedClients = this.clientData.filter(
      (client: { status: string }) => client.status === 'og-live'
    );
    this.loadScripts()
  }

  loadScripts(): void {
    this.loading = true;

    this.http.get(`${this.baseUrl}/commands?readOnly=false&enabled=true`).subscribe((data: any) => {
      this.scripts = data['hydra:member'];
      this.loading = false;
    }, (error) => {
      this.toastService.error(error.error['hydra:description']);
      this.loading = false;
    });
  }

  saveNewScript() {
    if (!this.newScript.trim()) {
      this.toastService.error('Debe ingresar un script antes de guardar.');
      return;
    }
    const dialogRef = this.dialog.open(SaveScriptComponent, {
      width: '400px',
      data: this.newScript
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        this.toastService.success('Script guardado correctamente');
      }
    });
  }

  toggleClientSelection(client: any) {
    client.selected = !client.selected;
    this.updateSelectedClients();
  }

  updateSelectedClients() {
    this.selectedClients = this.clientData.filter(
      (client: { selected: boolean; status: string }) => client.selected && client.status === "og-live"
    );
  }

  toggleSelectAll() {
    this.allSelected = !this.allSelected;
    this.clientData.forEach((client: { selected: boolean; status: string }) => {
      if (client.status === "og-live") {
        client.selected = this.allSelected;
      }
    });
  }

  getPartitionsTooltip(client: any): string {
    if (!client.partitions || client.partitions.length === 0) {
      return 'No hay particiones disponibles';
    }

    return client.partitions
      .map((p: { partitionNumber: any; size: any; filesystem: any }) => `#${p.partitionNumber} ${p.filesystem} - ${p.size / 1024 }GB`)
      .join('\n');
  }

  onScriptChange() {
    if (this.selectedScript) {
      this.scriptContent = this.selectedScript.script;

      const matches = this.scriptContent.match(/@(\w+)/g) || [];
      const uniqueParams = Array.from(new Set(matches.map(m => m.slice(1))));

      this.parameters = {};
      uniqueParams.forEach(param => this.parameters[param] = '');

      this.parameterNames = uniqueParams;

      this.updateScript();
    }
  }

  onParamChange(name: string, value: string): void {
    this.parameters[name] = value;
    this.updateScript();
  }

  updateScript(): void {
    let updatedScript = this.selectedScript.script;

    for (const [key, value] of Object.entries(this.parameters)) {
      const regex = new RegExp(`@${key}\\b`, 'g');
      updatedScript = updatedScript.replace(regex, value || `@${key}`);
    }

    this.scriptContent = updatedScript;
  }

  trackByIndex(index: number): number {
    return index;
  }


  save(): void {
    this.loading = true;

    this.http.post(`${this.baseUrl}/commands/run-script`, {
      clients: this.selectedClients.map((client: any) => client.uuid),
      script: this.commandType === 'existing'  ? this.scriptContent : this.newScript,
    }).subscribe(
      response => {
        this.toastService.success('Script ejecutado correctamente');
        this.dataChange.emit();
        this.router.navigate(['/commands-logs']);
      },
      error => {
        this.toastService.error('Error al ejecutar el script');
      }
    );

    this.loading = false;
  }
}
